unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Menus, ExtCtrls, EditPlugAPI, EditPlugAPITypes, TB2Item, TBX,
  TB2Dock, TB2Toolbar, ImgList, doscommand;

type
  TForm1 = class(TForm)
    ToolPan: TPanel;
    TBXToolbar1: TTBXToolbar;
    PopupMenu1: TPopupMenu;
    miAdd: TMenuItem;
    EditTool1: TMenuItem;
    Panel1: TPanel;
    Label1: TLabel;
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure miAddClick(Sender: TObject);
    procedure EditTool1Click(Sender: TObject);
    procedure ToolItemClick(Sender: TObject);
  private
    { Private declarations }
    FirstShow: boolean;
  public
    { Public declarations }
    procedure MakeButtonsFromIni;
  end;

type
  TTool = class
    Caption: string;
    Prog: string;
    Params: string;
    CaptureOutput: string;
    StartInDir: string;
    ParseRule: string;
    ScrollToLastLine: string;
    PauseAfterExec: string;
    constructor Create(ACaption: string; AProg: string; AParams: string;
      ACaptureOutput: string; AStartInDir: string; AParseRule: string;
      AScrollToLastLine: string; APauseAfterExec: string);
  end;

var
  Form1: TForm1;
  gPlugId: HINST;
  ToolList: TList;

implementation

{$R *.DFM}

uses filectrl, stringutils, fAddTool, fEditTools, inifiles, shellapi;

{----- EXPORTS ----------------------------------------------------------------}

procedure plgSetId(id: HINST); cdecl;
begin
  gPlugId:= id;
end; exports plgSetId;

function plgShow: HWND; cdecl;
begin
  if not assigned(form1) then begin
    form1:= tform1.create(nil);
    form1.show;
    epCreatePluginTab(gPlugId, form1.handle);
  end else begin
    form1.show;
  end;
  result:= form1.Handle;
end; exports plgShow;

procedure plgWinFree; cdecl;
begin
  if assigned(form1) then begin
    form1.Free;
    form1:=nil;
  end;
end; exports plgWinFree;

function plgGetWinHandle: HWND; cdecl;
begin
  result:=0;
  if assigned(form1) then result:= form1.Handle;
end; exports plgGetWinHandle;

{------------------------------------------------------------------------------}

constructor TTool.create(ACaption: string; AProg: string; AParams: string;
  ACaptureOutput: string; AStartInDir: string; AParseRule: string;
  AScrollToLastLine: string; APauseAfterExec: string);
begin
  Caption := ACaption;
  Prog := AProg;
  Params := AParams;
  CaptureOutput := ACaptureOutput;
  StartInDir := AStartInDir;
  ParseRule := AParseRule;
  ScrollToLastLine := AScrollToLastLine;
  PauseAfterExec := APauseAfterExec;
end;

procedure TForm1.MakeButtonsFromIni;
var
  ini: TIniFile;
  sl: TStringList;
  i: integer;
  btn: TTBXItem;
  tool: TTool;
  sprogram, sparams, scaptureoutput, sstartindir, sparserule, sscrolltolastline, spause: string;
  sec: string;
begin
  ini := TIniFile.create(IniPath);
  sl:= TStringList.create;
  try
    ini.ReadSections(sl);
    sl.sort;
    TBXToolbar1.Items.clear;
    ToolList.Clear;
    for i := 0 to sl.count-1 do begin
      sec := sl.strings[i];
      sprogram := ini.ReadString(sec, 'program', '');
      sparams := ini.ReadString(sec, 'params', '');
      scaptureoutput := ini.ReadString(sec, 'captureoutput', 'false');
      sstartindir := ini.ReadString(sec, 'startindir', '');
      sparserule := ini.ReadString(sec, 'parserule', '');
      sscrolltolastline := ini.ReadString(sec, 'scrolltolastline', '');
      spause := ini.ReadString(sec, 'pause', '');
      tool := TTool.create(sec, sprogram, sparams, scaptureoutput, sstartindir, sparserule, sscrolltolastline, spause);
      ToolList.Add(tool);
      btn := TTBXItem.create(TBXToolbar1);
      btn.Caption:= sec;
      btn.OnClick := ToolItemClick;
      TBXToolbar1.Items.Add(btn);
    end;
  finally
    ini.free;
    sl.free;
  end;
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  if FirstShow = true then begin
    FirstShow := false;
    MakeButtonsFromIni;
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  FirstShow := true;
end;

procedure TForm1.miAddClick(Sender: TObject);
begin
  fmAddToolTryCreate;
  fmAddTool.ShowModal;
end;

procedure TForm1.EditTool1Click(Sender: TObject);
begin
  fmEditToolsTryCreate;
  fmEditTools.ShowModal;
end;


function ExpandParamMacros(s: string): string;
var fname: string;
begin
  // %% = percent sign is changed temporarily to $$percent$$ to avoid coming across it in other searches
  result := StringReplace(s, '%%', '$$percent$$', [rfReplaceAll]);
  // %n = active editor filename
  result := StringReplace(result, '%n', epGetActiveEdFilenameAsString, [rfReplaceAll]);
  // %f = file name only
  result := StringReplace(result, '%f', ExtractFileName(epGetActiveEdFilenameAsString), [rfReplaceAll]);
  // %F = file name without extension
  fname := ExtractFileName(epGetActiveEdFilenameAsString);
  SetLength(fname,Length(fname)-Length(ExtractFileExt(epGetActiveEdFilenameAsString)));
  result := StringReplace(result, '%F', fname, [rfReplaceAll]);
  // %p = file path only
  result := StringReplace(result, '%p', ExtractFilePath(epGetActiveEdFilenameAsString), [rfReplaceAll]);
  // %e = file extension
  result := StringReplace(result, '%e', ExtractFileExt(epGetActiveEdFilenameAsString), [rfReplaceAll]);
  // %l = line number
  result := StringReplace(result, '%l', inttostr(epGetActiveEdLine), [rfReplaceAll]);
  // %c = column number
  result := StringReplace(result, '%c', inttostr(epGetActiveEdColumn), [rfReplaceAll]);
  // %w = word under cursor
  result := StringReplace(result, '%w', epGetActiveEdWordAsString, [rfReplaceAll]);
  // %w = word under cursor
  result := StringReplace(result, '%S', epGetActiveEdSelTextAsString, [rfReplaceAll]);
  // %% = percent sign
  result := StringReplace(result, '$$percent$$', '%', [rfReplaceAll]);
end;

procedure ExecCaptureProg(Prog: string; Params: string; StartInDir: string;
  ParseRule: string; scrolltolastline: boolean);
var cmd: string;
begin
  epClearOutputConsole(true);
  if not FileExists(Prog) then begin
    showmessage('The program: '+ prog+' cannot be found');
    exit;
  end;
  params := ExpandParamMacros(params);
  cmd := Prog+ ' "' + Params +'"';
  if (Length(StartInDir)=0) then StartInDir:= ExtractFilePath(epGetActiveEdFilenameAsString);
  if StartInDir = '' then StartInDir := 'c:\';
  if not DirectoryExists(StartInDir) then begin
    showmessage('"Start in" directory is invalid');
    exit;
  end;
  epConsoleExecCmd(pchar(cmd), pchar(StartInDir), pchar(ParseRule), scrolltolastline);
end;

function EdCmdPath: string;
begin
  result := epGetAppExePathAsString;
  result := ExtractFilePath(result);
  result := IncludeTrailingBackslash(result) + 'edcmd.exe';
end;

procedure ExecProgEdCmd(Prog: string; Params: string; StartInDir: string;
  PauseAfterExec: boolean);
var cmd: string;
begin
  if (Length(StartInDir)=0) then StartInDir:= ExtractFilePath(epGetActiveEdFilenameAsString);
  if StartInDir = '' then StartInDir := 'c:\';
  if not DirectoryExists(StartInDir) then begin
    showmessage('"Start in" directory is invalid');
    exit;
  end;
  params := ExpandParamMacros(params);
  cmd := '';
  if PauseAfterExec then cmd := cmd + '-p ';
  cmd := cmd + '"'+Prog+'" '+ Params;
  ShellExecute(0, 'open', PChar(EdCmdPath), PChar(cmd), PChar(StartInDir), SW_NORMAL);
end;

procedure ExecProg(Prog: string; Params: string; StartInDir: string);
begin
  if (Length(StartInDir)=0) then StartInDir:= ExtractFilePath(epGetActiveEdFilenameAsString);
  if StartInDir = '' then StartInDir := 'c:\';
  if not DirectoryExists(StartInDir) then begin
    showmessage('"Start in" directory is invalid');
    exit;
  end;
  params := ExpandParamMacros(params);
  ShellExecute(0, 'open', PChar(prog), PChar(Params), PChar(StartInDir), SW_NORMAL);
end;

procedure ToolExec(caption: string);
var
  i: integer;
  tool: TTool;
begin
  for i := 0 to ToolList.Count-1 do begin
    tool := TTool(ToolList.Items[i]);
    if tool.caption = caption then begin
      if strtobool(tool.CaptureOutput) then
        ExecCaptureProg(tool.prog, tool.params, tool.startindir, tool.parserule, strtobool(tool.scrolltolastline))
      else begin
        if strtobool(tool.PauseAfterExec) then
          ExecProgEdCmd(tool.prog, tool.params, tool.startindir, true)
        else
          ExecProg(tool.prog, tool.params, tool.startindir);
      end;
      break;
    end;
  end;
end;

procedure TForm1.ToolItemClick(Sender: TObject);
var btn: TTBXItem;
begin
  if sender is TTBXItem then begin
    btn := sender as TTBXItem;
    ToolExec(btn.caption);
  end;
end;

initialization
  ToolList := TList.create;
finalization
  ToolList.free;
end.
